import site
import os.path as path
from .SystemTools import *

class Setup():
    """Kraken_setup.
    """

    #rute = path.abspath(path.join(''))
    rute0 = site.getsitepackages()
    print("The library is installed in")
    print(rute + '/KrakenOS')
    print("for help see Examples directory")
    print("To change glass catalog modify catalog list in SetupCas.py")
    print("See User_Manual USER_MANUAL_KrakenOS_Provisional.pdf in:" )
    print(print(rute + '/Docs'))

    rute = rute0[0]
    ## Add catalog you need
    cat1 = (rute + '/KrakenOS/Cat/SCHOTT.AGF')
    cat2 = (rute + '/KrakenOS/Cat/TSPM.AGF')
    cat3 = (rute + '/KrakenOS/Cat/ESOPO.AGF')
    cat4 = (rute + '/KrakenOS/Cat/INFRARED.AGF')
    cat5 = (rute + '/KrakenOS/Cat/UTILIDADES.AGF')
    
    ## Add catalog to the list filepath
    filepath = [cat1, cat2, cat3, cat4, cat5]

    [CAT, NAMES, NM, ED, CD, TD, OD, LD, IT] = load_Catalog(filepath)
    file = (rute + '/KrakenOS/Cat/Alum.csv')

    (W_alum, N_alum, K_alum) = load_alluminum_complex(file)




# -*- coding: utf-8 -*-
"""
Created on Mon Jul  6 22:23:29 2020

@author: JOELHERRERAVAZQUEZ
"""

import glob

txtfiles = []
for file in glob.glob("*.AGF"):
    txtfiles.append(file)

print(txtfiles)




import numpy as np
import KrakenOS as Kos
import scipy

def R_RMS(delta_Z, L, M, N, X, Y):
    """R_RMS.

    Parameters
    ----------
    delta_Z :
        delta_Z
    L :
        L
    M :
        M
    N :
        N
    X :
        X
    Y :
        Y
    """
    X = (((L / N) * delta_Z) + X)
    Y = (((M / N) * delta_Z) + Y)
    cenX = np.mean(X)
    cenY = np.mean(Y)
    x1 = (X - cenX)
    y1 = (Y - cenY)
    R2 = ((x1 * x1) + (y1 * y1))
    R_RMS = np.sqrt(np.mean(R2))
    return R_RMS

def Phase(PUPIL):
    """Phase.

    Parameters
    ----------
    PUPIL :
        PUPIL
    """
    SYSTEM = PUPIL.SYSTEM
    sup = PUPIL.Surf
    W = PUPIL.W
    ApType = PUPIL.ApertureType
    ApVal = PUPIL.ApertureValue
    configuracion_1 = SYSTEM.SETUP
    Samp = PUPIL.Samp
    Ptype = PUPIL.Ptype
    FieldY = PUPIL.FieldY
    FieldX = PUPIL.FieldX
    FieldType = PUPIL.FieldType
    pSource_0 = [0.0, 0.0, 0.0]
    dCos = [0.0, 0.0, 1.0]
    W = W
    RR = Kos.raykeeper(SYSTEM)
    RR2 = Kos.raykeeper(SYSTEM)
    Pup = Kos.PupilCalc(SYSTEM, sup, W, ApType, ApVal)
    Pup = Kos.PupilCalc(SYSTEM, sup, W, ApType, ApVal)
    Pup.Samp = Samp
    Pup.Ptype = Ptype
    Pup.FieldY = FieldY
    Pup.FieldX = FieldX
    Pup.FieldType = FieldType
    (x, y, z, L, M, N) = Pup.Pattern2Field()
    SYSTEM.IgnoreVignetting(0)
    for i in range(0, len(x)):
        pSource_0 = [x[i], y[i], z[i]]
        dCos = [L[i], M[i], N[i]]
        SYSTEM.Trace(pSource_0, dCos, W)
        RR.push()
        pSource_0 = (np.asarray([x[i], y[i], z[i]]) * 0.01)
        dCos = [L[i], M[i], N[i]]
        SYSTEM.Trace(pSource_0, dCos, W)
        RR2.push()
    (X_P, Y_P, Z_P, L_P, M_P, N_P) = RR2.pick((- 1))
    delta_Z = 0
    ZZ = (L_P, M_P, N_P, X_P, Y_P)
    v = scipy.optimize.fsolve(R_RMS, delta_Z, args=ZZ)
    X_P = (((L_P / N_P) * v) + X_P)
    Y_P = (((M_P / N_P) * v) + Y_P)
    CenX = np.mean(X_P)
    CenY = np.mean(Y_P)
    CenZ = (np.mean(Z_P) - v)
    parax_focus = np.asarray([CenX, CenY, CenZ[0]])
    (XS, YS, ZS, LS, MS, NS) = RR.pick((- 1))
    top = np.squeeze(RR.valid_TOP)
    op = np.squeeze(RR.valid_OP)
    op = op[:, (- 1)]
    (XPUP, YPUP, ZPUP, LPUP, MPUP, NPUP) = RR.pick(sup)
    RR.clean()
    RR2.clean()
    [Px, Py, Pz] = Pup.PosPupOut
    (X, Y, Z) = (CenX, CenY, CenZ[0])
    (x0, y0, z0) = (Px, Py, Pz)
    X2 = ((X - x0) * (X - x0))
    Y2 = ((Y - y0) * (Y - y0))
    Z2 = ((Z - z0) * (Z - z0))
    S = np.sqrt(((X2 + Y2) + Z2))
    L = ((X - x0) / S)
    M = ((Y - y0) / S)
    N = ((Z - z0) / S)
    OPSR = S
    Pup.Ptype = 'chief'
    (xc, yc, zc, Lc, Mc, Nc) = Pup.Pattern2Field()
    SYSTEM.IgnoreVignetting()
    pSource_0 = [xc[0], yc[0], zc[0]]
    dCos = [Lc[0], Mc[0], Nc[0]]
    SYSTEM.Trace(pSource_0, dCos, W)
    [Xc, Yc, Zc] = SYSTEM.XYZ[(- 1)]
    [Lc, Mc, Nc] = SYSTEM.LMN[(- 1)]
    XYZ0 = [Xc, Yc, Zc]
    LMN0 = [Lc, Mc, Nc]
    AP = S
    P_O = Kos.surf()
    P_O.Rc = 0
    P_O.Thickness = Pz
    P_O.Glass = 'AIR'
    P_O.Diameter = 1.0
    P_P = Kos.surf()
    SYSTEM.Parax(W)
    P_P.Rc = np.abs(AP)
    P_P.Thickness = 0
    P_P.Diameter = ((P_P.Rc * 0.99) * 2.0)
    P_P.Glass = 'MIRROR'
    P_P.DespX = Px
    P_P.DespY = Py
    P_P.TiltX = np.rad2deg(np.arcsin((- Mc)))
    P_P.TiltY = np.rad2deg(np.arcsin((Lc / np.cos(np.arcsin((- Mc))))))
    P_P.AxisMove = 0
    P_P.Order = 1
    P_I = Kos.surf()
    P_I.Diameter = (((P_P.Rc * 0.99) * 2.0) * 10.0)
    B = [P_O, P_P]
    PS = Kos.system(B, configuracion_1)
    RR = Kos.raykeeper(PS)
    PS.Trace(XYZ0, LMN0, W)
    OPSR = PS.TOP
    for i in range(0, len(XS)):
        pSource_0 = [XS[i], YS[i], ZS[i]]
        dCos = [LS[i], MS[i], NS[i]]
        PS.Trace(pSource_0, dCos, W)
        RR.push()
    VT = np.squeeze(RR.valid_TOP)
    VTOPSR = OPSR
    RRR = (((((VT - VTOPSR) * 1000.0) / W) * Pup.RadPupOut) / Pup.RadPupInp)
    RRRR = 1000.0
    P2V = (np.min(RRR) - np.max(RRR))
    return ((YPUP / 1000.0), (XPUP / 1000.0), RRR, P2V)





import csv
import numpy as np

def load_alluminum_complex(file):
    """load_alluminum_complex.

    Parameters
    ----------
    file :
        file
    """
    w = []
    n = []
    k = []
    with open(file, 'rt') as f:
        csv_reader = csv.reader(f, delimiter=';')
        next(csv_reader)
        for line in csv_reader:
            w.append(float(line[0]))
            n.append(float(line[1]))
            k.append(float(line[2]))
    return (w, n, k)

def load_Catalog(FileCat):
    """load_Catalog.

    Parameters
    ----------
    FileCat :
        FileCat
    """
    cat = []
    ARR_CAT = []
    ARR_NM = []
    ARR_ED = []
    ARR_CD = []
    ARR_TD = []
    ARR_OD = []
    ARR_LD = []
    ARR_IT = []
    print('Loading glass calatogs:')
    for file in FileCat:
        ARR_CAT.append(file)
        f = open(file, 'r')
        for x in f:
            cat.append(x)
    con = 0
    coords = []
    names = []
    for f in cat:
        cadena = f.split()
        cad = cadena[0]
        if (cad == 'NM'):
            coords.append(con)
            names.append(cadena[1])
        con = (con + 1)
    names = np.asarray(names)
    for i in range(0, (len(coords) - 1)):
        ITT = []
        for j in range(coords[i], coords[(i + 1)]):
            cadena = cat[j].split()
            cad = cat[j][2:].split()
            if (cadena[0] == 'NM'):
                NM = cad
            if (cadena[0] == 'ED'):
                ED = cad
            if (cadena[0] == 'CD'):
                CD = cad
            if (cadena[0] == 'TD'):
                TD = cad
            if (cadena[0] == 'OD'):
                OD = cad
            if (cadena[0] == 'LD'):
                LD = cad
            if (cadena[0] == 'IT'):
                IT = cad
                ITT.append(IT)
        NM = np.asarray(NM[1:(- 1)], dtype=np.float64)
        ED = np.asarray(ED, dtype=np.float64)
        CD = np.asarray(CD, dtype=np.float64)
        TD = np.asarray(TD, dtype=np.float64)
        OD = np.asarray(OD)
        LD = np.asarray(LD, dtype=np.float64)
        IT = np.asarray(ITT, dtype=np.float64).T
        ARR_NM.append(NM)
        ARR_ED.append(ED)
        ARR_CD.append(CD)
        ARR_TD.append(TD)
        ARR_OD.append(OD)
        ARR_LD.append(LD)
        ARR_IT.append(IT)
    CATALOG = [ARR_CAT, names, ARR_NM, ARR_ED, ARR_CD, ARR_TD, ARR_OD, ARR_LD, ARR_IT]
    return CATALOG




# !/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Examp-Tel_2M_Atmospheric_Refraction_Corrector"""

import KrakenOS as Kos
import numpy as np
import matplotlib.pyplot as plt
import scipy

# _________________________________________________________________#


# _________________________________________________________________#
P_Obj = Kos.surf()
P_Obj.Rc = 0
P_Obj.Thickness = 1000 + 3452.2
P_Obj.Glass = "AIR"
P_Obj.Diameter = 1059.0 * 2.0
P_Obj.Drawing = 0
# _________________________________________________________________#
Thickness = 3452.2
M1 = Kos.surf()
M1.Rc = -9638.0
M1.Thickness = -Thickness
M1.k = -1.07731
M1.Glass = "MIRROR"
M1.Diameter = 1059.0 * 2.0
M1.InDiameter = 250 * 2.0
# _________________________________________________________________#
M2 = Kos.surf()
M2.Rc = -3.93E+003
M2.Thickness = Thickness + 1037.525 - 300.0
M2.k = -4.3281
M2.Glass = "MIRROR"
M2.Diameter = 336.5 * 2.0
M2.AxisMove = 0
# _________________________________________________________________#
C1 = Kos.surf()
C1.Thickness = 5
C1.Glass = "BK7"
C1.Diameter = 100

C2 = Kos.surf()
C2.Thickness = C1.Thickness
C2.Glass = "F2"
C2.Diameter = 100
C2.TiltY = 1.55
C2.AxisMove = 0
E = C2.TiltY * 100.0

C3 = Kos.surf()
C3.Thickness = C1.Thickness + 288.631
C3.Glass = "AIR"
C3.Diameter = 100
C3.TiltY = -0.0
C3.AxisMove = 0

# -------------------------------------

C6 = Kos.surf()
C6.Thickness = C1.Thickness
C6.Glass = "AIR"
C6.Diameter = 100

# _________________________________________________________________#
P_Ima = Kos.surf()
P_Ima.Diameter = 100.0
P_Ima.Glass = "AIR"
P_Ima.Name = "Plano imagen"

# _________________________________________________________________#


A = [P_Obj, M1, M2, C1, C2, C3, P_Ima]
configuracion_1 = Kos.Setup()
Telescopio = Kos.system(A, configuracion_1)

A = [P_Obj, M1, M2, P_Ima]
Telescope = Kos.system(A, configuracion_1)
Rayos = Kos.raykeeper(Telescope)

W = 0.60169
sup = 1  # Defining M1 as enter pupil diameter
AperVal = 2000
AperType = "EPD"
Pup = Kos.PupilCalc(Telescopio, sup, W, AperType, AperVal)
Pup.Samp = 5
Pup.FieldType = "angle"
Pup.Ptype = "hexapolar"
Pup.FieldX = 0.0

xa, ya, za, La, Ma, Na = Pup.Pattern2Field()
Kos.TraceLoop(xa, ya, za, La, Ma, Na, W, Rayos)
X, Y, Z, L, M, N = Rayos.pick(-1)

v = Kos.BestFocus(X, Y, Z, L, M, N)
rms, center_x, center_y = Kos.RMS(X, Y, Z, L, M, N)
print(rms)

M2.Thickness = M2.Thickness + v
Telescope.SetData()
Rayos.clean()

Kos.TraceLoop(xa, ya, za, La, Ma, Na, W, Rayos)
X, Y, Z, L, M, N = Rayos.pick(-1)
rms, center_x, center_y = Kos.RMS(X, Y, Z, L, M, N)
print(rms)

# plt.plot(X,Y, 'x', c="b")
# plt.xlabel('x')
# plt.ylabel('y')
# plt.title('Spot Diagram')
# plt.axis('square')
# plt.show()


# Rayos1=Kos.raykeeper(Telescopio)
# Rayos2=Kos.raykeeper(Telescopio)
# Rayos3=Kos.raykeeper(Telescopio)
# Rayos=Kos.raykeeper(Telescopio)

# W = 0.60169
# sup = 1 # Difining M1 as enter pupil diameter
# AperVal = 2000
# AperType = "EPD" # "STOP"
# Pup = Kos.PupilCalc(Telescopio, sup, W, AperType, AperVal)
# Pup.Samp=5
# Pup.FieldType = "angle"

# Pup.AtmosRef = 1
# Pup.T   = 283.15    # k
# Pup.P   = 101300    # Pa
# Pup.H   = 0.5       # Humidity ratio 1 to 0
# Pup.xc  = 400       # ppm
# Pup.lat = 31        # degrees
# Pup.h   = 2800      # meters
# Pup.l1  = W  # micron
# Pup.l2  = 0.50169   # micron
# Pup.z0  = 55.0      # degrees

# Pup.Ptype = "hexapolar"
# Pup.FieldX = 0.0


# W1 = 0.50169
# Pup.l2  = W1
# xa,ya,za,La,Ma,Na=Pup.Pattern2Field()

# W2 = 0.60169
# Pup.l2  = W2
# xb,yb,zb,Lb,Mb,Nb=Pup.Pattern2Field()

# W3 = 0.70169
# Pup.l2  = W3
# xc,yc,zc,Lc,Mc,Nc=Pup.Pattern2Field()

# ############################################


# for i in range(0,len(xc)):
#     pSource_0 = [xc[i], yc[i], zc[i]]
#     dCos=[Lc[i], Mc[i], Nc[i]]
#     Telescopio.Trace(pSource_0, dCos, W3)
#     Rayos3.push()
#     Rayos.push()

# for i in range(0,len(xb)):
#     pSource_0 = [xb[i], yb[i], zb[i]]
#     dCos=[Lb[i], Mb[i], Nb[i]]
#     Telescopio.Trace(pSource_0, dCos, W2)
#     Rayos2.push()
#     Rayos.push()


# for i in range(0,len(xa)):
#     pSource_0 = [xa[i], ya[i], za[i]]
#     dCos=[La[i], Ma[i], Na[i]]
#     Telescopio.Trace(pSource_0, dCos, W1)
#     Rayos1.push()
#     Rayos.push()


# ############################################

# # Kos.display2d(Telescopio,Rayos,1)

# X,Y,Z,L,M,N=Rayos1.pick(-1)
# plt.plot(X*1000.0,Y*1000.0+E, 'x', c="b")


# X,Y,Z,L,M,N=Rayos3.pick(-1)
# plt.plot(X*1000.0,Y*1000.0+E, 'x', c="g")

# X,Y,Z,L,M,N=Rayos2.pick(-1)
# plt.plot(X*1000.0,Y*1000.0+E, 'x', c="r")


# # axis labeling
# plt.xlabel('x')
# plt.ylabel('y')

# # figure name
# plt.title('Spot Diagram')
# plt.axis('square')
# plt.ylim(151,159)
# plt.show()


# v = Kos.BestFocus(X,Y,Z,L,M,N)
# rms = Kos.RMS(X,Y,Z,L,M,N)

# print("sol ---------------------")
# print(v)




import numpy as np
from .SurfTools import surface_tools as SUT

class Hit_Solver():
    """Hit_Solver.
    """


    def __init__(self, SurfData):
        """__init__.

        Parameters
        ----------
        SurfData :
            SurfData
        """
        self.SDT = SurfData
        self.n = len(self.SDT)
        self.SuTo = SUT(SurfData)
        self.vj = 0
        self.vevaX = 0
        self.vevaY = 0
        self.NP_x1 = 0
        self.NP_y1 = 0
        self.NP_z1 = 0
        self.MN = 0
        self.LN = 0
        self.SuTo.ErrSurfCase = 0

    def __Fxyz(self, x, y, z):
        """__Fxyz.

        Parameters
        ----------
        x :
            x
        y :
            y
        z :
            z
        """
        VF = (self.SuTo.SurfaceShape(x, y, self.vj) - z)
        return VF

    def SurfDer(self, x, y, z):
        """SurfDer.

        Parameters
        ----------
        x :
            x
        y :
            y
        z :
            z
        """
        delta = 0.0001
        vx = np.asarray([(x + delta), x, x, (x - delta), x, x])
        vy = np.asarray([y, (y + delta), y, y, (y - delta), y])
        vz = np.asarray([z, z, (z + delta), z, z, (z - delta)])
        V = self.__Fxyz(vx, vy, vz)
        Dx = ((V[0] - V[3]) / (2.0 * delta))
        Dy = ((V[1] - V[4]) / (2.0 * delta))
        Dz = ((V[2] - V[5]) / (2.0 * delta))
        Dr = np.sqrt((((Dx ** 2) + (Dy ** 2)) + (Dz ** 2)))
        return ((Dx / Dr), (Dy / Dr), (Dz / Dr))

    def __lineCurve(self, vz):
        """__lineCurve.

        Parameters
        ----------
        vz :
            vz
        """
        PX = (((vz - self.NP_z1) * self.LN) + self.NP_x1)
        PY = (((vz - self.NP_z1) * self.MN) + self.NP_y1)
        self.vevaX = PX
        self.vevaY = PY
        Vzf = (self.SuTo.SurfaceShape(PX, PY, self.vj) - vz)
        return Vzf

    def __DerLineCurve(self, gf):
        """__DerLineCurve.

        Parameters
        ----------
        gf :
            gf
        """
        delta = 1e-13
        Dz = ((self.__lineCurve((gf + delta)) - self.__lineCurve((gf - delta))) / (2.0 * delta))
        return Dz

    def SolveHit(self, Px1, Py1, Pz1, L, M, N, j):
        """SolveHit.

        Parameters
        ----------
        Px1 :
            Px1
        Py1 :
            Py1
        Pz1 :
            Pz1
        L :
            L
        M :
            M
        N :
            N
        j :
            j
        """
        if (len(self.SDT[j].Error_map) != 0):
            self.SuTo.ErrSurfCase = 1
        if (len(self.SDT[j].Error_map) == 0):
            self.SuTo.ErrSurfCase = 0
        self.vevaX = 0
        self.vevaY = 0
        self.vj = j
        cnt = 0
        PP_z2 = Pz1
        P_z2 = Pz1
        self.NP_x1 = Px1
        self.NP_y1 = Py1
        self.NP_z1 = Pz1
        self.MN = (M / N)
        self.LN = (L / N)
        while True:
            FdeX = self.__lineCurve(PP_z2)
            DerFdeX = self.__DerLineCurve(PP_z2)
            PP_z2 = (PP_z2 - (FdeX / DerFdeX))
            if (np.abs((PP_z2 - P_z2)) < 1e-05):
                break
            else:
                P_z2 = PP_z2
            if (cnt == 30):
                break
            cnt = (cnt + 1)
        P_z2 = PP_z2
        P_x2 = (((P_z2 - self.NP_z1) * self.LN) + self.NP_x1)
        P_y2 = (((P_z2 - self.NP_z1) * self.MN) + self.NP_y1)
        self.SuTo.ErrSurfCase = 0
        if (len(self.SDT[j].Error_map) != 0):
            PP_z2 = P_z2
            cnt = 0
            while True:
                FdeX = self.__lineCurve(PP_z2)
                DerFdeX = self.__DerLineCurve(PP_z2)
                PP_z2 = (PP_z2 - (FdeX / DerFdeX))
                if (np.abs((PP_z2 - P_z2)) < 1e-05):
                    break
                else:
                    P_z2 = PP_z2
                if (cnt == 3):
                    break
                cnt = (cnt + 1)
            P_z2 = PP_z2
            P_x2 = (((P_z2 - self.NP_z1) * self.LN) + self.NP_x1)
            P_y2 = (((P_z2 - self.NP_z1) * self.MN) + self.NP_y1)
        return (P_x2, P_y2, P_z2)





import numpy as np
import pyvista as pv
import matplotlib.pyplot as plt

def wavelength_to_rgb(wavelength, gamma=1.0):
    """wavelength_to_rgb.

    Parameters
    ----------
    wavelength :
        wavelength
    gamma :
        gamma
    """
    wavelength = float(wavelength)
    if (380 <= wavelength <= 440):
        attenuation = (0.3 + ((0.7 * (wavelength - 380)) / (440 - 380)))
        R = ((((- (wavelength - 440)) / (440 - 380)) * attenuation) ** gamma)
        G = 0.0
        B = ((1.0 * attenuation) ** gamma)
    elif (440 <= wavelength <= 490):
        R = 0.0
        G = (((wavelength - 440) / (490 - 440)) ** gamma)
        B = 1.0
    elif (490 <= wavelength <= 510):
        R = 0.0
        G = 1.0
        B = (((- (wavelength - 510)) / (510 - 490)) ** gamma)
    elif (510 <= wavelength <= 580):
        R = (((wavelength - 510) / (580 - 510)) ** gamma)
        G = 1.0
        B = 0.0
    elif (580 <= wavelength <= 645):
        R = 1.0
        G = (((- (wavelength - 645)) / (645 - 580)) ** gamma)
        B = 0.0
    elif (645 <= wavelength <= 750):
        attenuation = (0.3 + ((0.7 * (750 - wavelength)) / (750 - 645)))
        R = ((1.0 * attenuation) ** gamma)
        G = 0.0
        B = 0.0
    else:
        R = 0.0
        G = 0.0
        B = 0.0
    R *= 255
    G *= 255
    B *= 255
    return [(int(R) / 255.0), (int(G) / 255.0), (int(B) / 255.0)]

def display3d(SYSTEM, RAYS, view=0):
    """display3d.

    Parameters
    ----------
    SYSTEM :
        SYSTEM
    RAYS :
        RAYS
    view :
        view
    """
    p = pv.Plotter(shape=(1, 1), notebook=False)
    Absorb_color = np.array([(10 / 256), (23 / 256), (24 / 256)])
    Mirror_color = np.array([(189 / 256), (189 / 256), (189 / 256)])
    Glass_color = np.array([(12 / 256), (238 / 256), (246 / 256)])
    recorte = view
    NN = SYSTEM.AAA.n_blocks
    if (SYSTEM.SDT[0].Drawing == 0):
        points2 = np.c_[(0, 0, 0)]
        c = pv.PolyData(points2)
        cc = pv.PolyData(points2)
    if (SYSTEM.SDT[0].Drawing == 1):
        c = SYSTEM.AAA[0]
        cc = SYSTEM.AAA[0]
    for n in range(1, NN):
        if (SYSTEM.SDT[n].Drawing == 1):
            AAAA = SYSTEM.AAA[n]
            if (SYSTEM.SDT[n].Glass != 'NULL'):
                if (SYSTEM.SDT[n].Color == [0, 0, 0]):
                    if (SYSTEM.SDT[n].Glass == 'MIRROR'):
                        color = Mirror_color
                    else:
                        color = Glass_color
                    if (SYSTEM.SDT[n].Glass == 'ABSORB'):
                        color = Absorb_color
                else:
                    color = SYSTEM.SDT[n].Color
                cc = cc.merge(AAAA)
                if (recorte == 1):
                    clippedx = AAAA.clip((1, 0, 0), invert=False)
                    clippedy = clippedx.clip((0, 1, 0), invert=False)
                    c = c.merge(clippedy)
                    clippedx = AAAA.clip(((- 1), 0, 0), invert=False)
                    clippedy = clippedx.clip((0, (- 1), 0), invert=False)
                    c = c.merge(clippedy)
                    clippedx = AAAA.clip((1, 0, 0), invert=False)
                    clippedy = clippedx.clip((0, (- 1), 0), invert=False)
                    c = c.merge(clippedy)
                if (recorte == 0):
                    No_clipped = AAAA
                    c = c.merge(No_clipped)
                if (recorte == 2):
                    clippedx = AAAA.clip((1, 0, 0), invert=False)
                    c = c.merge(clippedx)
                p.add_mesh(c, color, opacity=0.95, specular=1, specular_power=15, smooth_shading=True, show_edges=False)
                edges = c.extract_feature_edges(boundary_edges=True, feature_edges=True, manifold_edges=False)
                p.add_mesh(edges, 'red')
                points2 = np.c_[(0, 0, 0)]
                c = pv.PolyData(points2)
    p.add_mesh(SYSTEM.DDD, color=[5, 5, 5], opacity=0.95, show_edges=None)
    NN = SYSTEM.AAA.n_blocks
    n = 0
    for g in SYSTEM.side_number:
        if (SYSTEM.SDT[g].Drawing == 1):
            if (SYSTEM.SDT[g].Color == [0, 0, 0]):
                if (SYSTEM.SDT[g].Glass == 'MIRROR'):
                    LL_color = Mirror_color
                else:
                    LL_color = Glass_color
                if (SYSTEM.SDT[g].Glass == 'ABSORB'):
                    LL_color = Absorb_color
                color = LL_color
            else:
                color = SYSTEM.SDT[g].Color
            if (recorte == 1):
                clippedx = SYSTEM.BBB[n].clip('x', invert=False)
                p.add_mesh(clippedx, color, smooth_shading=True, show_edges=None)
                clippedy = SYSTEM.BBB[n].clip('-y', invert=False)
                p.add_mesh(clippedy, color, smooth_shading=True, show_edges=None)
            if (recorte == 0):
                No_clipped = SYSTEM.BBB[n]
                p.add_mesh(No_clipped, color, smooth_shading=False, show_edges=None)
            if (recorte == 2):
                BBBB = SYSTEM.BBB[n]
                clippedx = BBBB.clip((1, 0, 0), invert=False)
                p.add_mesh(clippedx, color, smooth_shading=True, show_edges=None)
        n = (n + 1)
    if (len(RAYS.RayWave) != 0):
        RW = np.asarray(RAYS.RayWave)
        for i in range(0, np.shape(RW)[0]):
            RGB = wavelength_to_rgb((RW[i] * 1000.0))
            p.add_mesh(RAYS.CCC[i], color=RGB, opacity=0.95, smooth_shading=True, line_width=1.0, show_edges=None)
    p.add_axes()
    p.set_focus([0, 0, 0])
    p.set_position([(- 1000), 0, 0])
    p.set_viewup([0, 1, 0])
    p.enable_anti_aliasing()
    p.disable_anti_aliasing()
    p.enable_eye_dome_lighting()
    p.disable_eye_dome_lighting()
    p.set_background('white', top='royalblue')
    p.add_text('Kraken - Optical Simulator')
    p.show_grid(font_size=4)
    p.show(auto_close=False, interactive=True, interactive_update=False)

def display2d(SYSTEM, RAYS, view=0):
    """display2d.

    Parameters
    ----------
    SYSTEM :
        SYSTEM
    RAYS :
        RAYS
    view :
        view
    """
    fs=11
    plt.figure()
    plt.rcParams['font.family'] = 'Times New Roman'
    L_color = [0, 2, 3]
    recorte = view
    NN = SYSTEM.AAA.n_blocks
    if (SYSTEM.SDT[0].Drawing == 0):
        points2 = np.c_[(0, 0, 0)]
        c = pv.PolyData(points2)
    if (SYSTEM.SDT[0].Drawing == 1):
        c = SYSTEM.AAA[0]
    for n in range(0, NN):
        if (SYSTEM.SDT[n].Drawing == 1):
            AAAA = SYSTEM.AAA[n]
            if (SYSTEM.SDT[n].Glass != 'NULL'):
                (PosX, PosY) = SYSTEM.SDT[n].Nm_Poss
                s = SYSTEM.SDT[n].Name
                ss = SYSTEM.Object_Num[n]
                if (view == 0):
                    LT = ''
                    (ax, ay, az) = edge_3d(AAAA, 1, 0, 0)
                    (az, ay) = filter_face_2dplot(az, ay)
                    plt.plot(az, ay, LT, c='black', linewidth=0.5)
                    (ax, ay, az) = edge_3d(AAAA, (- 1), 0, 0)
                    (az, ay) = filter_face_2dplot(az, ay)
                    plt.plot(az, ay, LT, c='black', linewidth=0.5)

                    plt.text(((np.max(az) + PosX) + 1), ((np.max(ay) + PosY) - 1), s, fontsize=fs)
                    delta = ((np.max(ay) - np.min(ay)) / 10)
                    plt.text(az[np.argmin((ay * ay))], (np.min(ay) - (1.5 * delta)), (('[' + str(ss)) + ']'), fontsize=fs)
                    plt.plot([az[np.argmin((ay * ay))], az[np.argmin((ay * ay))]], [0, (np.min(ay) - delta)], '-.', c='red', linewidth=0.5)
                    if ((PosX != 0) or (PosY != 0)):
                        plt.arrow((np.max(az) + PosX), (np.max(ay) + PosY), (- PosX), (- PosY), head_width=0.5, head_length=1.0, fc='k', ec='k', length_includes_head=True)
                if (view == 1):
                    LT = ''
                    (ax, ay, az) = edge_3d(AAAA, 0, 1, 0)
                    (az, ax) = filter_face_2dplot(az, ax)
                    plt.plot(az, ax, LT, c='black', linewidth=0.5)
                    (ax, ay, az) = edge_3d(AAAA, 0, (- 1), 0)
                    (az, ax) = filter_face_2dplot(az, ax)
                    plt.plot(az, ax, LT, c='black', linewidth=0.5)
                    delta = ((np.max(ax) - np.min(ax)) / 10)

                    plt.text(((np.max(az) + PosX) + 1), ((np.max(ax) + PosY) - 1), s, fontsize=fs)
                    plt.text(az[np.argmin((ax * ax))], (np.min(ax) - (1.5 * delta)), (('[' + str(ss)) + ']'), fontsize=fs)
                    plt.plot([az[np.argmin((ax * ax))], az[np.argmin((ax * ax))]], [0, (np.min(ax) - delta)], '-.', c='red', linewidth=0.5)
                    if ((PosX != 0) or (PosY != 0)):
                        plt.arrow((np.max(az) + PosX), (np.max(ax) + PosY), (- PosX), (- PosY), head_width=0.5, head_length=1.0, fc='k', ec='k', length_includes_head=True)
    NN = SYSTEM.BBB.n_blocks
    for n in range(0, NN):
        TT = SYSTEM.BBB[n]
        sim = '-.'
        if (view == 0):
            (ax, ay, az) = edge_3d(TT, 1, 0, 0)
            plt.plot(az, ay, sim, c='black', linewidth=0.5)
            (ax, ay, az) = edge_3d(AAAA, (- 1), 0, 0)
            plt.plot(az, ay, sim, c='black', linewidth=0.5)
        if (view == 1):
            (ax, ay, az) = edge_3d(TT, 0, 1, 0)
            plt.plot(az, ax, sim, c='black', linewidth=0.5)
            (ax, ay, az) = edge_3d(AAAA, 0, (- 1), 0)
            plt.plot(az, ax, sim, c='black', linewidth=0.5)
    if (len(RAYS.RayWave) != 0):
        RW = np.asarray(RAYS.RayWave)
        for i in range(0, np.shape(RW)[0]):
            RGB = wavelength_to_rgb((RW[i] * 1000.0))
            RRR = RAYS.CCC[i]
            Ax = RRR.points[:, 0]
            Ay = RRR.points[:, 1]
            Az = RRR.points[:, 2]
            if (view == 0):
                plt.plot(Az, Ay, color=RGB, linewidth=0.5)
            if (view == 1):
                plt.plot(Az, Ax, color=RGB, linewidth=0.5)
    plt.title('System Plot')
    plt.xlabel('Z')
    if (view == 0):
        plt.ylabel('Y')
    if (view == 1):
        plt.ylabel('X')
    plt.axis('equal')
    plt.show()

def edge_3d(MeshObject, cx, cy, xz):
    """edge_3d.

    Parameters
    ----------
    MeshObject :
        MeshObject
    cx :
        cx
    cy :
        cy
    xz :
        xz
    """
    c = MeshObject.clip((cx, cy, xz), invert=False)
    edges = c.extract_feature_edges(boundary_edges=True, feature_edges=False, manifold_edges=False)
    Ax = edges.points[:, 0]
    Ay = edges.points[:, 1]
    Az = edges.points[:, 2]
    Ax = np.asarray(Ax)
    Ay = np.asarray(Ay)
    Az = np.asarray(Az)
    Xe = []
    Ye = []
    Ze = []
    if (cx == 1):
        i = np.argmin(Ax)
    if (cx == (- 1)):
        i = np.argmax(Ax)
    if (cy == 1):
        i = np.argmin(Ay)
    if (cy == (- 1)):
        i = np.argmax(Ay)
    x0 = Ax[i]
    y0 = Ay[i]
    z0 = Az[i]
    for j in range(0, (np.shape(Ax)[0] - 1)):
        AAx = Ax[i]
        AAy = Ay[i]
        AAz = Az[i]
        Xe.append(AAx)
        Ye.append(AAy)
        Ze.append(AAz)
        Ax = np.delete(Ax, i)
        Ay = np.delete(Ay, i)
        Az = np.delete(Az, i)
        X = (Ax - AAx)
        Y = (Ay - AAy)
        Z = (Az - AAz)
        R = np.sqrt((((X * X) + (Y * Y)) + (Z * Z)))
        i = np.argmin(R)
    Xe.append(x0)
    Ye.append(y0)
    Ze.append(z0)
    Xe = np.asarray(Xe)
    Ye = np.asarray(Ye)
    Ze = np.asarray(Ze)
    return (Xe, Ye, Ze)

def filter_face_2dplot(v1, v2):
    """filter_face_2dplot.

    Parameters
    ----------
    v1 :
        v1
    v2 :
        v2
    """
    av1 = np.copy(v1)
    av2 = np.copy(v2)
    av1 = np.roll(av1, (- 1))
    av2 = np.roll(av2, (- 1))
    yy = (v1 - av1)
    zz = (v2 - av2)
    R = np.sqrt(((yy * yy) + (zz * zz)))
    M = np.mean(R)
    AW = np.argwhere((R < M))
    v1 = v1[AW]
    v2 = v2[AW]
    av1 = np.copy(v1)
    av2 = np.copy(v2)
    av1 = np.roll(av1, 1)
    av2 = np.roll(av2, 1)
    yy = (v1 - av1)
    zz = (v2 - av2)
    R = np.sqrt(((yy * yy) + (zz * zz)))
    AW = np.argmax(R)
    v1 = np.roll(v1, (- AW))
    v2 = np.roll(v2, (- AW))
    return (v1, v2)





import numpy as np
from .Physics import *

class snell_refraction_vector_physics():
    """snell_refraction_vector_physics.
    """


    def __init__(self):
        """__init__.
        """
        pass

    def calculate(self, s1_norm, Nsurf_norm, n1, n2, empty1, empty2, empty3, empty4, Secuen):
        """calculate.

        Parameters
        ----------
        s1_norm :
            s1_norm
        Nsurf_norm :
            Nsurf_norm
        n1 :
            n1
        n2 :
            n2
        empty1 :
            empty1
        empty2 :
            empty2
        empty3 :
            empty3
        empty4 :
            empty4
        Secuen :
            Secuen
        """
        s1_norm = np.asarray(s1_norm)
        Nsurf_norm = np.asarray(Nsurf_norm)
        cos = (np.dot(Nsurf_norm, s1_norm) / (np.linalg.norm(Nsurf_norm) * np.linalg.norm(s1_norm)))
        if (cos < (- 1.0)):
            ang = 180
        else:
            ang = np.rad2deg(np.arccos(cos))
        if (ang <= 90.0):
            Nsurf_norm = (- Nsurf_norm)
        Nsurf_Cros_s1 = np.cross(Nsurf_norm, s1_norm)
        SIGN = 1
        if (n2 == (- 1)):
            n2 = (- n1)
            SIGN = (- 1)
        NN = (n1 / n2)
        R = ((NN * NN) * np.dot(Nsurf_Cros_s1, Nsurf_Cros_s1))
        if (Secuen == 1):
            R = 2.0
        if (R > 1):
            n2 = (- n1)
            NN = (n1 / n2)
            R = ((NN * NN) * np.dot(Nsurf_Cros_s1, Nsurf_Cros_s1))
            SIGN = (- 1)
        s2 = ((NN * np.cross(Nsurf_norm, np.cross((- Nsurf_norm), s1_norm))) - (Nsurf_norm * np.sqrt((1.0 - ((NN * NN) * np.dot(Nsurf_Cros_s1, Nsurf_Cros_s1))))))
        orizontal = [0, 0, 1]
        cos = (np.dot(orizontal, s2) / (np.linalg.norm(orizontal) * np.linalg.norm(s2)))
        ang = np.rad2deg(np.arccos(cos))
        return (s2, np.abs(n2), SIGN)

class paraxial_exact_physics():
    """paraxial_exact_physics.
    """


    def __init__(self):
        """__init__.
        """
        pass

    def calculate(self, s1_norm, Nsurf_norm, n1, n2, empty1, empty2, empty3, empty4, empty5):
        """calculate.

        Parameters
        ----------
        s1_norm :
            s1_norm
        Nsurf_norm :
            Nsurf_norm
        n1 :
            n1
        n2 :
            n2
        empty1 :
            empty1
        empty2 :
            empty2
        empty3 :
            empty3
        empty4 :
            empty4
        empty5 :
            empty5
        """
        return (Nsurf_norm, 1, 1)

class diffraction_grating_physics():
    """diffraction_grating_physics.
    """


    def __init__(self):
        """__init__.
        """
        pass

    def calculate(self, S, R, N, Np, D, Ord, d, W, Secuent):
        """calculate.

        Parameters
        ----------
        S :
            S
        R :
            R
        N :
            N
        Np :
            Np
        D :
            D
        Ord :
            Ord
        d :
            d
        W :
            W
        Secuent :
            Secuent
        """
        lamb = W
        RefRef = ((- 1) * np.sign(Np))
        SIGN = 1
        print(RefRef, Np)
        if (Np == (- 1)):
            Np = np.abs(N)
        mu = (N / Np)
        T = ((Ord * lamb) / (Np * d))
        V = (mu * np.dot(S, R))
        W = ((((mu * mu) - 1) + (T * T)) - (((2.0 * mu) * T) * np.dot(S, D)))
        Q = ((RefRef * np.sqrt(((V * V) - W))) - V)
        S = np.asarray(S)
        Sp = (((mu * S) - (T * D)) + (Q * R))
        Sp = (Sp / np.linalg.norm(Sp))
        return (Sp, np.abs(Np), SIGN)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import glob

myFiles = glob.glob('*.py')

for i in myFiles:
    if i == "TestAll.py" or i == "Examp_MultiCore.py" or i == "Examp_RonchiTest.py":
        print("---------",i, "No se ejecuta")
    else:
        print("-------------------")
        print(i)
        exec(open(i).read())




from setuptools import setup

setup(
    name='KrakenOS',
    version='0.0.23',
    packages=['KrakenOS', 'KrakenOS.AstroAtmosphere', 'KrakenOS.Cat', 'KrakenOS.Examples', 'KrakenOS.Docs'],
    package_data={'': ['LICENSE.txt', '../*.AGF', '../*.agf', '../*.stl', '../*.scad', '../*.pdf']},
    include_package_data=True,
    url='https://github.com/Garchupiter/Kraken-Optical-Simulator',
    license='GNU General Public License v3.0',
    author='joel Herrera et al.',
    author_email='joel@astro.unam.mx',
    description='Optical Simulation and ray tracing '
)
